-- Getter/Setter functions for main configuration data

local function L(x) return Translations.Equalizer.L(x) end 

Equalizer.Datamodel = {}

function Equalizer.Datamodel.getWatchlistStrings()
	local strings={}
	if Equalizer_Settings.debug then 
		print ("building watch list for Role " .. Equalizer_Settings.currentRole)
	end
	for i,v in ipairs(Equalizer_Settings.role[Equalizer_Settings.currentRole]) do
		str=v.buff
		if str:sub(1, 1) ~= "-" then
			if v.target ~= "self" then
				str = str .. " " .. L("on") .. " " .. v.target
			end
			if v.stacks > 1 then
				str = str .. " (" .. v.stacks .. L("stacks") ..")"
			end
		end
		if Equalizer_Settings.debug then
			print ("adding "..i..": "..str)
		end
		table.insert(strings, str)
	end
	table.insert(strings, L("<empty>"))
	return strings
end

function Equalizer.Datamodel.getWatchlistCount()
	return #Equalizer_Settings.role[Equalizer_Settings.currentRole]
end

local function parametersOk(buff, stacks, target, ability)
	if buff == nil or buff=="" then print "buff is nil" return false end
	if stacks == nil or stacks=="" then print "stacks is nil" return false end
	if target == nil or target=="" then print "target is nil" return false end
	if ability == nil or ability=="" then print "ability is nil" return false end
	return true
end

function Equalizer.Datamodel.addWatchList(buff, stacks, target, ability)
	if buff:sub(1, 1) == "-" then
		stacks=1
		target="--"
		ability="--"
	end
	if not parametersOk(buff, stacks, target, ability) then return end
	-- print ("adding buff "..buff.." stacks "..stacks.." target "..target.." ability "..ability)
	
	table.insert(Equalizer_Settings.role[Equalizer_Settings.currentRole], {
		["buff"] = buff, ["stacks"] = tonumber(stacks),
		["target"] = target, ["ability"] = ability
	})
	local index=#Equalizer_Settings.role[Equalizer_Settings.currentRole]
	if  Equalizer_Session["knownAbilities"][ability]
	and Equalizer_Session["knownAbilities"][ability]["icon"]
	then 
		Equalizer_Settings.role[Equalizer_Settings.currentRole][index]["icon"]=
			Equalizer_Session["knownAbilities"][ability]["icon"]
	end
	Equalizer.UI.triggerWatchlistChanged()
end

function Equalizer.Datamodel.setWatchList(index, buff, stacks, target, ability)
	if buff:sub(1, 1) == "-" then
		stacks=1
		target="--"
		ability="--"
	end
	if not parametersOk(buff, stacks, target, ability) then return end
	-- print ("setting index "..index..": buff "..buff.." stacks "..stacks.." target "..target.." ability "..ability)
	
	Equalizer_Settings.role[Equalizer_Settings.currentRole][index]={
		["buff"] = buff, ["stacks"] = tonumber(stacks),
		["target"] = target, ["ability"] = ability
	}
	if  Equalizer_Session["knownAbilities"][ability]
	and Equalizer_Session["knownAbilities"][ability]["icon"]
	then 
		Equalizer_Settings.role[Equalizer_Settings.currentRole][index]["icon"]=
			Equalizer_Session["knownAbilities"][ability]["icon"]
	end
	Equalizer.UI.triggerWatchlistChanged()
end

function Equalizer.Datamodel.deleteWatchListIndex(i) 
	table.remove(Equalizer_Settings.role[Equalizer_Settings.currentRole], i)
	Equalizer.UI.triggerWatchlistChanged()
end

function Equalizer.Datamodel.moveItemUp(i)
	if i>1 and i<=#Equalizer_Settings.role[Equalizer_Settings.currentRole] then
		temp=Equalizer_Settings.role[Equalizer_Settings.currentRole][i-1]
		Equalizer_Settings.role[Equalizer_Settings.currentRole][i-1]=
		Equalizer_Settings.role[Equalizer_Settings.currentRole][i]
		Equalizer_Settings.role[Equalizer_Settings.currentRole][i]=temp
		Equalizer.UI.triggerWatchlistChanged()
		return true
	else
		return false
	end
end

function Equalizer.Datamodel.moveItemDown(i)
	if i<#Equalizer_Settings.role[Equalizer_Settings.currentRole] then
		temp=Equalizer_Settings.role[Equalizer_Settings.currentRole][i+1]
		Equalizer_Settings.role[Equalizer_Settings.currentRole][i+1]=
		Equalizer_Settings.role[Equalizer_Settings.currentRole][i]
		Equalizer_Settings.role[Equalizer_Settings.currentRole][i]=temp
		Equalizer.UI.triggerWatchlistChanged()
		return true
	else
		return false
	end
end

function Equalizer.Datamodel.getWatchListIndex(i)
	if not Equalizer_Settings.role[Equalizer_Settings.currentRole][i] then
		return { buff = "<new>", stacks = 1, target = "", ability = "", isEmpty = true }
	else
		return Equalizer_Settings.role[Equalizer_Settings.currentRole][i]
	end
end
