local AddonData, PrivateTable = ...

-- Main file

local function L(x) return Translations.Equalizer.L(x) end 

local function ListAbilities()
	abilities = Inspect.Ability.New.List()
	details = Inspect.Ability.New.Detail(abilities)
	table.sort(details)
	for k,v in pairs(details) do
		print (k .. " _ " .. v.name)
	end
end

local function ListBuffs(unit)
	print "-----------------------"
	details = Inspect.Unit.Detail(unit)
	if details then
		print ("Unit Name: "..details.name)
	end
	buffs = Inspect.Buff.List(unit)
	if not buffs then
		print ("no buffs on "..unit)
		return
	end
	details = Inspect.Buff.Detail(unit, buffs)
	table.sort(details)
	for k,v in pairs(details) do
		print (k .. " _ " .. v.name)
	end
end

local function setRole(newRole)
	print (L("Using role ")..newRole)
	-- print ("old role was "..Equalizer_Settings.currentRole)
	Equalizer_Settings.currentRole=newRole
	if not Equalizer_Settings.role[newRole] then
		Equalizer_Settings.role[newRole]={}
	end
	Equalizer.UI.triggerWatchlistChanged()
	Equalizer.UI.triggerRoleChanged()
end

local function toggleDebug()
	if (Equalizer_Settings.debug) then
		Equalizer_Settings.debug = false
		print ("Debugging turned off")
	else
		Equalizer_Settings.debug = true
		print ("Debugging turned on")
	end
end

local doUpdates = true
function Equalizer.updateUI()
	if doUpdates then
		-- prevent a failing periodicUpdate function from being called over and over.
		doUpdates=false
		Equalizer.UI.periodicUpdate()
		doUpdates=true
	end
end

function Equalizer.SlashHandler(args)
	local usage = ": /eq role <role> | config | hide | size <n> | height <n> | reset | update | freeze | melt | lock | unlock | hideothers | showothers | alpha <n> | vertical | horizontal | copy <role>"
	local r = {}
	local numargs = 1
	for token in string.gmatch(args, "[^%s]+") do
		r[numargs] = token
		numargs=numargs+1
	end
	if numargs>1 then
		if r[1] == "abil" then
			ListAbilities()
		elseif r[1] == "buffs" then
			if r[2]==nil then
				print(L("Usage: /eq buffs <target>, e.g. /eq buffs player"));
			else
				ListBuffs(r[2])
			end
		elseif r[1] == "role" then
			if r[2]==nil then
				print(L("Usage: /eq role <name/number>, e.g. /eq role 2"));
			else
				setRole(r[2])
			end
		elseif r[1] == "config" then
			Equalizer.UI.showConfigScreen()
		elseif r[1] == "hide" then
			Equalizer.UI.hideConfigScreen()
		elseif r[1] == "size" and r[2] and tonumber(r[2]) > 3 then
			Equalizer_Settings.buttonsize=tonumber(r[2])
			Equalizer.UI.triggerLayoutChanged()
		elseif r[1] == "height" and r[2] and tonumber(r[2]) > 3 then
			Equalizer_Settings.barheight=tonumber(r[2])
			Equalizer.UI.triggerLayoutChanged()
		elseif r[1] == "debug" then
			toggleDebug()
		elseif r[1] == "update" then
			Equalizer.updateUI()
		elseif r[1] == "freeze" then			-- reduce resource consumption
			doUpdates=false
		elseif r[1] == "melt" then				-- update again
			doUpdates=true
		elseif r[1] == "lock" then
			Equalizer_Settings.locked=true
		elseif r[1] == "unlock" then
			Equalizer_Settings.locked=false
		elseif r[1] == "hideothers" then
			Equalizer_Settings.othersAlpha=0
		elseif r[1] == "showothers" then
			Equalizer_Settings.othersAlpha = tonumber(r[2]) or 0.2
		elseif r[1] == "alpha" then
			Equalizer_Settings.bgAlpha = tonumber(r[2]) or 0
			Equalizer.UI.triggerAlphaChanged()
		elseif r[1] == "vertical" then
			Equalizer_Settings.vertical = true
			Equalizer.UI.triggerLayoutChanged()
		elseif r[1] == "horizontal" then
			Equalizer_Settings.vertical = false
			Equalizer.UI.triggerLayoutChanged()
		elseif r[1] == "reset" then
			Equalizer.BuffsReset()
		elseif r[1] == "copy" and r[2] and Equalizer_Settings.role[r[2]] then
			local nr=r[2]
			local cr=Equalizer_Settings.currentRole
			Equalizer_Settings.role[cr]={}
			for i, v in pairs(Equalizer_Settings.role[nr]) do
				Equalizer_Settings.role[cr][i]={}
				Equalizer_Settings.role[cr][i].ability=v.ability
				Equalizer_Settings.role[cr][i].buff   =v.buff   
				Equalizer_Settings.role[cr][i].icon   =v.icon   
				Equalizer_Settings.role[cr][i].target =v.target 
				Equalizer_Settings.role[cr][i].stacks =v.stacks 
			end
			Equalizer.UI.triggerWatchlistChanged()
			print(L("Settings copied from role ")..r[2])
		else
			print (L("Usage") .. usage);
		end
	else
		print (L("Usage") .. usage);
	end
end

local buffseen={}
function Equalizer.BuffAdded(unit, buffs)
	-- print ("buff added to "..unit)
	changed = false
	for k,v in pairs(buffs) do
		if not buffseen[k] then
			buffseen[k]=true
			-- print (k .. " _ " .. (v and "true" or "false"))
			detail=Inspect.Buff.Detail(unit, k)
			if (not Equalizer_Session["knownBuffs"][detail.name]) then
				Equalizer_Session["knownBuffs"][detail.name]=detail
				changed = true
			end
		end
	end
	if (changed) then
		Equalizer.UI.setListOfKnownBuffs(Equalizer_Session["knownBuffs"])
	end
end

function Equalizer.BuffsReset()
	Equalizer_Session["knownBuffs"]={}
	Equalizer.BuffAdded("player", Inspect.Buff.List("player"))
end

function Equalizer.AbilityAdded(abilities)
	-- print ("buff added to "..unit)
	changed = false
	for k,v in pairs(abilities) do
		detail=Inspect.Ability.New.Detail(k)
		-- print (k .. " + " .. (v and "true" or "false"))
		if (detail
		    and not Equalizer_Session["knownAbilities"][detail.name]) then
			Equalizer_Session["knownAbilities"][detail.name]=detail
			changed = true
		end
	end
	if (changed) then
		Equalizer.UI.setListOfKnownAbilities(Equalizer_Session["knownAbilities"])
	end
end

function Equalizer.AbilityRemoved(abilities)
	-- print ("buff added to "..unit)
	changed = false
	for k,v in pairs(abilities) do
		detail=Inspect.Ability.New.Detail(k)
		-- print (k .. " - " .. (v and "true" or "false"))
		if (detail
		    and Equalizer_Session["knownAbilities"][detail.name]) then
			Equalizer_Session["knownAbilities"][detail.name]=nil
			changed = true
		end
	end
	if (changed) then
		Equalizer.UI.setListOfKnownAbilities(Equalizer_Session["knownAbilities"])
	end
end

function Equalizer.enterSecure()
	Equalizer.secureMode = true
	Equalizer.UI.secureModeEntered()
end

function Equalizer.leaveSecure()
	Equalizer.secureMode = false
	Equalizer.UI.secureModeExited()
end


table.insert(Command.Slash.Register("eq"),	{ Equalizer.SlashHandler, "Equalizer", "SlashHandler" })
table.insert(Event.Buff.Add,            { Equalizer.BuffAdded, "Equalizer", "BuffAdded" })
table.insert(Event.Ability.New.Add,     { Equalizer.AbilityAdded, "Equalizer", "AbilityAdded" })
table.insert(Event.Ability.New.Remove,  { Equalizer.AbilityRemoved, "Equalizer", "AbilityRemoved" })
table.insert(Event.System.Update.Begin, { Equalizer.updateUI, "Equalizer", "SystemUpdate" })
table.insert(Event.System.Secure.Enter, { Equalizer.enterSecure, "Equalizer", "EnterSecure" })
table.insert(Event.System.Secure.Leave, { Equalizer.leaveSecure, "Equalizer", "LeaveSecure" })
